precision highp float;

varying vec2 v_texCoord;
varying vec2 v_colorUV;

uniform sampler2D u_texture;
uniform sampler2D u_maskMap;

uniform float u_opacity;

uniform sampler2D maskMap;

uniform float maskWidth;
uniform float maskHeight;
uniform vec2 center;

void main() {
    vec4 color = texture2D(u_texture, v_colorUV);
    vec4 mask = texture2D(u_maskMap, v_texCoord);

    // maskWidth > 0 && maskHeight > 0
    float hasMask = step(0.0, maskWidth) * step(0.0, maskHeight);
    float maskHasZeroRadius = mod(step(0.0, maskWidth) * step(maskWidth, 0.0) + 1.0, 2.0) * mod(step(0.0, maskHeight) * step(maskHeight, 0.0) + 1.0, 2.0);
    hasMask = hasMask * maskHasZeroRadius;
    
    // v_texCoord.x > center.x - maskWidth/2.0 && v_texCoord.x < center.x + maskWidth/2.0 &&
    // v_texCoord.y > center.y - maskHeight/2.0 && v_texCoord.y < center.y + maskHeight/2.0
    float isInMaskBounds = step((center.x - maskWidth/2.0), v_texCoord.x) * mod(step((center.x + maskWidth/2.0), v_texCoord.x) + 1.0, 2.0) *
                            step((center.y - maskHeight/2.0), v_texCoord.y) * mod(step((center.y + maskHeight/2.0), v_texCoord.y) + 1.0, 2.0);
    
    float useMask = hasMask * isInMaskBounds;
    
    vec2 mask_Coord = vec2((v_texCoord.x - center.x + maskWidth/2.0)/maskWidth, (v_texCoord.y - center.y + maskHeight/2.0)/maskHeight) * useMask;
    vec4 holeMask = texture2D(maskMap, mask_Coord);
    float maskMultiplier = max(min(1.0, holeMask.a), 0.0);
    
    maskMultiplier = useMask*maskMultiplier + mod(useMask + 1.0, 2.0);
    gl_FragColor = vec4(color.rgb, mask.a * u_opacity * color.a * maskMultiplier);
}
