precision highp float;

varying vec2 v_texCoord;

uniform sampler2D u_texture;
uniform float u_opacity;
uniform vec3 u_rgbHues;
uniform vec3 u_cmyHues;
uniform vec3 u_rgbSaturation;
uniform vec3 u_cmySaturation;
uniform vec3 u_color;

void main() {
    vec4 color = texture2D(u_texture, v_texCoord);
    
    float maxColor = max(color.r, color.g);
    maxColor = max(color.b, maxColor);
    
    float minColor = min(color.r, color.g);
    minColor = min(color.b, minColor);
    
    float delta = maxColor - minColor;
    
    float redIsMax = step(maxColor, color.r) * step(color.r, maxColor);
    float greenIsMax = step(maxColor, color.g) * step(color.g, maxColor) * mod(redIsMax - 1.0, 2.0);
    float blueIsMax = step(maxColor, color.b) * step(color.b, maxColor) * mod(redIsMax - 1.0, 2.0) * mod(greenIsMax - 1.0, 2.0);
    float originalHue = redIsMax * (mod((color.g - color.b)/delta, 6.0) * 60.0) + greenIsMax * (60.0 * (((color.b - color.r)/delta) + 2.0)) + blueIsMax * (60.0 * (((color.r - color.g)/delta) + 4.0));
    
    // originalHue < 30.0 || originalHue >= 330.0 -> Red Pixel
    float isRedPixel = mod(min(max(originalHue - 330.0, 0.0), 1.0) + min(max(30.0 - originalHue, 0.0), 1.0), 2.0);
    // original Hue in [30, 90) -> Yellow Pixel
    float isYellowPixel = min(max(originalHue - 30.0, 0.0), 1.0) * min(max(90.0 - originalHue, 0.0), 1.0);
    // original Hue in [90, 150) -> Green Pixel
    float isGreenPixel = min(max(originalHue - 90.0, 0.0), 1.0) * min(max(150.0 - originalHue, 0.0), 1.0);
    // original Hue in [150, 210) -> Cyan Pixel
    float isCyanPixel = min(max(originalHue - 150.0, 0.0), 1.0) * min(max(210.0 - originalHue, 0.0), 1.0);
    // original Hue in [210, 270) -> Blue Pixel
    float isBluePixel = min(max(originalHue - 210.0, 0.0), 1.0) * min(max(270.0 - originalHue, 0.0), 1.0);
    // original Hue in [270, 330) -> Magenta Pixel
    float isMagentaPixel = min(max(originalHue - 270.0, 0.0), 1.0) * min(max(330.0 - originalHue, 0.0), 1.0);
    
    float hueShift = isRedPixel*u_rgbHues.r + isYellowPixel*u_cmyHues.b + isGreenPixel*u_rgbHues.g + isCyanPixel*u_cmyHues.r + isBluePixel*u_rgbHues.b + isMagentaPixel*u_cmyHues.g;
    float S = isRedPixel*u_rgbSaturation.r + isYellowPixel*u_cmySaturation.b + isGreenPixel*u_rgbSaturation.g + isCyanPixel*u_cmySaturation.r + isBluePixel*u_rgbSaturation.b + isMagentaPixel*u_cmySaturation.g;
    S = 1.0 + S/100.0;
    float V = 1.0;
    float VSU = V*S*cos(-hueShift);
    float VSW = V*S*sin(-hueShift);
    
    float r = (0.299*V + 0.701*VSU + 0.168*VSW) * color.r + (0.587*V - 0.587*VSU + 0.330*VSW) * color.g + (0.114*V - 0.114*VSU - 0.497*VSW) * color.b;
    float g = (0.299*V - 0.299*VSU - 0.328*VSW) * color.r + (0.587*V + 0.413*VSU + 0.035*VSW) * color.g + (0.114*V - 0.114*VSU + 0.292*VSW) * color.b;
    float b = (0.299*V - 0.300*VSU + 1.250*VSW) * color.r + (0.587*V - 0.588*VSU - 1.050*VSW) * color.g + (0.114*V + 0.886*VSU - 0.203*VSW) * color.b;
    gl_FragColor = vec4(r * u_color.r, g * u_color.g, b * u_color.b, color.a * u_opacity);
}